#vider la mémoire
rm(list=ls())

#générer un jeu de données
n <- 1000
p <- 5000
set.seed(1)
df <- data.frame(matrix(rnorm(n*p),nrow=n,ncol=p))

#noms des variables attribués par défaut
colnames(df)

#fonction pour traitement d'un vecteur
ftrans <- function(x){
  #calcul des quartiles
  q <- quantile(x,probs=c(0.25,0.5,0.75))
  #remplacement de la valeur par le quartile le plus proche
  #on aurait pu passer aussi par une fonction vectorisée
  res <- sapply(x,function(v){q[which.min((v-q)^2)]})
  names(res) <- c()
  #vecteur est renvoyé
  return(res)
}

#appliquer avec une boucle for
fun_boucle <- function(df){
  #liste vide
  df_res <- list()
  #complétée au fur et à mesure
  for (j in 1:ncol(df)){
    df_res[[j]] <- ftrans(df[,j])
  }
  #renvoyer sous forme de data.frame
  return(data.frame(df_res))
}

#appliquer avec une boucle
print(system.time(dfRes1 <- fun_boucle(df)))
sum_1 <- sapply(dfRes1,sum)

#fonction pour un lapply
fun_lapply <- function(df){
  #lapply renvoie une liste -> castée en data.frame
  return(data.frame(lapply(df,ftrans)))
}

#appliquer à notre df avec un lapply
print(system.time(dfRes2 <- fun_lapply(df)))
sum_2 <- sapply(dfRes2,sum)

#vérifier les sommes de contrôle
print(sum((sum_1-sum_2)^2))

#parallélisation "parallel"
library(parallel)

#fonction avec parLapply
fun_parlapply <- function(df){
  #initialisation des workers
  clust <- parallel::makeCluster(spec=6)
  #lancer les calculs - idem lapply()
  df_res <- data.frame(parLapply(clust,df,ftrans))
  #stopper les workers
  parallel::stopCluster(clust)
  #renvoyer le résultat
  return(df_res)
}

#avec parlapply
print(system.time(dfRes3 <- fun_parlapply(df)))
sum_3 <- sapply(dfRes3,sum)

#vérif.
print(sum((sum_1-sum_3)^2))

#autres librairies de parallélisation
library(doParallel)

#fonction dopar standard
fun_dopar <- function(df){
  #initialiser et enregistrer les moteurs -- attention cf. la doc.
  doParallel::registerDoParallel(cores=6)
  #lancer la boucle - res est une liste
  res <- foreach(col=df,.export=c("ftrans")) %dopar% {return(ftrans(col))}
  #stopper les moteurs
  doParallel::stopImplicitCluster()
  #renvoyer sous forme de data.frame
  return(data.frame(res))
}

#avec foreach
print(system.time(dfRes4 <- fun_dopar(df)))
sum_4 <- sapply(dfRes4,sum)

#vérif.
print(sum((sum_1-sum_4)^2))

